# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'client.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt5 import QtCore, QtWidgets


class UiMainClientWindow(object):
    """ Класс основного окна клиента. """
    def __init__(self):
        self.label_contacts = None
        self.central_widget = None
        self.btn_add_contact = None
        self.btn_remove_contact = None
        self.label_history = None
        self.text_message = None
        self.label_new_message = None
        self.list_contacts = None
        self.list_messages = None
        self.btn_send = None
        self.btn_clear = None
        self.menubar = None
        self.menu = None
        self.menu_2 = None
        self.statusBar = None
        self.menu_exit = None
        self.menu_add_contact = None
        self.menu_del_contact = None

    def setup_ui(self, main_client_window):
        main_client_window.setObjectName('main_client_window')
        main_client_window.resize(852, 565)
        main_client_window.setMinimumSize(QtCore.QSize(756, 534))
        self.central_widget = QtWidgets.QWidget(main_client_window)
        self.central_widget.setObjectName('central_widget')
        self.label_contacts = QtWidgets.QLabel(self.central_widget)
        self.label_contacts.setGeometry(QtCore.QRect(160, 10, 101, 16))
        self.label_contacts.setObjectName('label_contacts')
        self.btn_add_contact = QtWidgets.QPushButton(self.central_widget)
        self.btn_add_contact.setGeometry(QtCore.QRect(20, 30, 131, 131))
        self.btn_add_contact.setObjectName('btn_add_contact')
        self.btn_remove_contact = QtWidgets.QPushButton(self.central_widget)
        self.btn_remove_contact.setGeometry(QtCore.QRect(20, 170, 131, 111))
        self.btn_remove_contact.setObjectName('btn_remove_contact')
        self.label_history = QtWidgets.QLabel(self.central_widget)
        self.label_history.setGeometry(QtCore.QRect(390, 10, 431, 21))
        self.label_history.setObjectName('label_history')
        self.text_message = QtWidgets.QTextEdit(self.central_widget)
        self.text_message.setGeometry(QtCore.QRect(390, 430, 441, 91))
        self.text_message.setObjectName('text_message')
        self.label_new_message = QtWidgets.QLabel(self.central_widget)
        self.label_new_message.setGeometry(QtCore.QRect(390, 410, 441, 16))
        self.label_new_message.setObjectName('label_new_message')
        self.list_contacts = QtWidgets.QListView(self.central_widget)
        self.list_contacts.setGeometry(QtCore.QRect(160, 30, 211, 491))
        self.list_contacts.setObjectName('list_contacts')
        self.list_messages = QtWidgets.QListView(self.central_widget)
        self.list_messages.setGeometry(QtCore.QRect(390, 30, 441, 371))
        self.list_messages.setObjectName('list_messages')
        self.btn_send = QtWidgets.QPushButton(self.central_widget)
        self.btn_send.setGeometry(QtCore.QRect(20, 410, 131, 111))
        self.btn_send.setObjectName('btn_send')
        self.btn_clear = QtWidgets.QPushButton(self.central_widget)
        self.btn_clear.setGeometry(QtCore.QRect(20, 290, 131, 111))
        self.btn_clear.setObjectName('btn_clear')
        main_client_window.setCentralWidget(self.central_widget)
        self.menubar = QtWidgets.QMenuBar(main_client_window)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 852, 22))
        self.menubar.setObjectName('menubar')
        self.menu = QtWidgets.QMenu(self.menubar)
        self.menu.setObjectName('menu')
        self.menu_2 = QtWidgets.QMenu(self.menubar)
        self.menu_2.setObjectName('menu_2')
        main_client_window.setMenuBar(self.menubar)
        self.statusBar = QtWidgets.QStatusBar(main_client_window)
        self.statusBar.setObjectName('statusBar')
        main_client_window.setStatusBar(self.statusBar)
        self.menu_exit = QtWidgets.QAction(main_client_window)
        self.menu_exit.setObjectName('menu_exit')
        self.menu_add_contact = QtWidgets.QAction(main_client_window)
        self.menu_add_contact.setObjectName('menu_add_contact')
        self.menu_del_contact = QtWidgets.QAction(main_client_window)
        self.menu_del_contact.setObjectName('menu_del_contact')
        self.menu.addAction(self.menu_exit)
        self.menu_2.addAction(self.menu_add_contact)
        self.menu_2.addAction(self.menu_del_contact)
        self.menu_2.addSeparator()
        self.menubar.addAction(self.menu.menuAction())
        self.menubar.addAction(self.menu_2.menuAction())

        self.retranslate_ui(main_client_window)
        self.btn_clear.clicked.connect(self.text_message.clear)
        QtCore.QMetaObject.connectSlotsByName(main_client_window)

    def retranslate_ui(self, main_client_window):
        _translate = QtCore.QCoreApplication.translate
        main_client_window.setWindowTitle(
            _translate('main_client_window', 'Чат программа alpha release'))
        self.label_contacts.setText(_translate('main_client_window',
                                               'Контакты'))
        self.btn_add_contact.setText(_translate('main_client_window',
                                                'Добавить'))
        self.btn_remove_contact.setText(_translate('main_client_window',
                                                   'Удалить контакт'))
        self.label_history.setText(_translate('main_client_window', 'История'))
        self.label_new_message.setText(_translate('main_client_window',
                                                  'Отправить новое сообщение'))
        self.btn_send.setText(_translate('main_client_window', 'Отправить'))
        self.btn_clear.setText(_translate('main_client_window', 'Очистить'))
        self.menu.setTitle(_translate('main_client_window', 'Файл'))
        self.menu_2.setTitle(_translate('main_client_window', 'Контакты'))
        self.menu_exit.setText(_translate('main_client_window', 'Выход'))
        self.menu_add_contact.setText(_translate('main_client_window',
                                                 'Добавить контакт'))
        self.menu_del_contact.setText(_translate('main_client_window',
                                                 'Удалить контакт'))
